@@ -70,8 +70,8 @@ bool OCLTypeToSPIRVLegacy::runOnModule(Module &M) {
   return runOCLTypeToSPIRV(M);
 }
 
-OCLTypeToSPIRVBase OCLTypeToSPIRVPass::run(llvm::Module &M,
-                                           llvm::ModuleAnalysisManager &MAM) {
+OCLTypeToSPIRVBase &OCLTypeToSPIRVPass::run(llvm::Module &M,
+                                            llvm::ModuleAnalysisManager &MAM) {
   runOCLTypeToSPIRV(M);
   return *this;
 }
@@ -82,6 +82,8 @@ bool OCLTypeToSPIRVBase::runOCLTypeToSPIRV(Module &Module) {
   LLVM_DEBUG(dbgs() << "Enter OCLTypeToSPIRV:\n");
   M = &Module;
   Ctx = &M->getContext();
+  AdaptedTy.clear();
+  WorkSet.clear();
   auto Src = getSPIRVSource(&Module);
   if (std::get<0>(Src) != spv::SourceLanguageOpenCL_C)
     return false;
@@ -104,11 +106,12 @@ bool OCLTypeToSPIRVBase::runOCLTypeToSPIRV(Module &Module) {
   return false;
 }
 
-void OCLTypeToSPIRVBase::addAdaptedType(Value *V, Type *T) {
+void OCLTypeToSPIRVBase::addAdaptedType(Value *V, Type *Ty,
+                                        unsigned AddrSpace) {
   LLVM_DEBUG(dbgs() << "[add adapted type] ";
              V->printAsOperand(dbgs(), true, M);
-             dbgs() << " => " << *T << '\n');
-  AdaptedTy[V] = T;
+             dbgs() << " => " << *Ty << '\n');
+  AdaptedTy[V] = {Ty, AddrSpace};
 }
 
 void OCLTypeToSPIRVBase::addWork(Function *F) {
@@ -117,34 +120,6 @@ void OCLTypeToSPIRVBase::addWork(Function *F) {
   WorkSet.insert(F);
 }
 
-/// Find index of \param V as argument of function call \param CI.
-static unsigned getArgIndex(CallInst *CI, Value *V) {
-  for (unsigned AI = 0, AE = CI->arg_size(); AI != AE; ++AI) {
-    if (CI->getArgOperand(AI) == V)
-      return AI;
-  }
-  llvm_unreachable("Not argument of function call");
-  return ~0U;
-}
-
-/// Find index of \param V as argument of function call \param CI.
-static unsigned getArgIndex(Function *F, Value *V) {
-  auto A = F->arg_begin(), E = F->arg_end();
-  for (unsigned I = 0; A != E; ++I, ++A) {
-    if (&(*A) == V)
-      return I;
-  }
-  llvm_unreachable("Not argument of function");
-  return ~0U;
-}
-
-/// Get i-th argument of a function.
-static Argument *getArg(Function *F, unsigned I) {
-  auto AI = F->arg_begin();
-  std::advance(AI, I);
-  return &(*AI);
-}
-
 /// Create a new function type if \param F has arguments in AdaptedTy, and
 /// propagates the adapted arguments to functions called by \param F.
 void OCLTypeToSPIRVBase::adaptFunction(Function *F) {
@@ -158,15 +133,17 @@ void OCLTypeToSPIRVBase::adaptFunction(Function *F) {
     auto Loc = AdaptedTy.find(&I);
     auto Found = (Loc != AdaptedTy.end());
     Changed |= Found;
-    ArgTys.push_back(Found ? Loc->second : I.getType());
+    ArgTys.push_back(Found ? Loc->second.first : I.getType());
 
     if (Found) {
-      for (auto U : I.users()) {
-        if (auto CI = dyn_cast<CallInst>(U)) {
-          auto ArgIndex = getArgIndex(CI, &I);
+      auto *Ty = Loc->second.first;
+      unsigned AddrSpace = Loc->second.second;
+      for (auto &U : I.uses()) {
+        if (auto *CI = dyn_cast<CallInst>(U.getUser())) {
+          auto ArgIndex = CI->getArgOperandNo(&U);
           auto CF = CI->getCalledFunction();
           if (AdaptedTy.count(CF) == 0) {
-            addAdaptedType(getArg(CF, ArgIndex), Loc->second);
+            addAdaptedType(CF->getArg(ArgIndex), Ty, AddrSpace);
             addWork(CF);
           }
         }
@@ -179,7 +156,7 @@ void OCLTypeToSPIRVBase::adaptFunction(Function *F) {
 
   auto FT = F->getFunctionType();
   FT = FunctionType::get(FT->getReturnType(), ArgTys, FT->isVarArg());
-  addAdaptedType(F, FT);
+  addAdaptedType(F, FT, 0);
 }
 
 // Handle functions with sampler arguments that don't get called by
@@ -204,13 +181,10 @@ void OCLTypeToSPIRVBase::adaptArgumentsBySamplerUse(Module &M) {
           AdaptedTy.count(SamplerArg) != 0) // Already traced this, move on.
         continue;
 
-      if (isSPIRVType(SamplerArg->getType(), kSPIRVTypeName::Sampler))
-        return;
-
-      addAdaptedType(SamplerArg, getSamplerType(&M));
+      addAdaptedType(SamplerArg, getSamplerStructType(&M), SPIRAS_Constant);
       auto Caller = cast<Argument>(SamplerArg)->getParent();
       addWork(Caller);
-      TraceArg(Caller, getArgIndex(Caller, SamplerArg));
+      TraceArg(Caller, cast<Argument>(SamplerArg)->getArgNo());
     }
   };
 
@@ -221,7 +195,12 @@ void OCLTypeToSPIRVBase::adaptArgumentsBySamplerUse(Module &M) {
     StringRef DemangledName;
     if (!oclIsBuiltin(MangledName, DemangledName, false))
       continue;
-    if (DemangledName.find(kSPIRVName::SampledImage) == std::string::npos)
+    // Note: kSPIRVName::ConvertHandleToSampledImageINTEL contains
+    // kSPIRVName::SampledImage as a substring, but we still want to continue in
+    // this case.
+    if (DemangledName.find(kSPIRVName::SampledImage) == std::string::npos ||
+        DemangledName.find(kSPIRVName::ConvertHandleToSampledImageINTEL) !=
+            std::string::npos)
       continue;
 
     TraceArg(&F, 1);
@@ -233,20 +212,28 @@ void OCLTypeToSPIRVBase::adaptFunctionArguments(Function *F) {
   if (TypeMD)
     return;
   bool Changed = false;
-  auto FT = F->getFunctionType();
-  auto PI = FT->param_begin();
   auto Arg = F->arg_begin();
-  for (unsigned I = 0; I < F->arg_size(); ++I, ++PI, ++Arg) {
-    auto NewTy = *PI;
-    if (isPointerToOpaqueStructType(NewTy)) {
-      auto STName = NewTy->getPointerElementType()->getStructName();
+  SmallVector<StructType *, 4> ParamTys;
+  getParameterTypes(F, ParamTys);
+
+  // If we couldn't get any information from demangling, there is nothing that
+  // can be done.
+  if (ParamTys.empty())
+    return;
+
+  for (unsigned I = 0; I < F->arg_size(); ++I, ++Arg) {
+    StructType *NewTy = ParamTys[I];
+    if (NewTy && NewTy->isOpaque()) {
+      auto STName = NewTy->getStructName();
       if (!hasAccessQualifiedName(STName))
         continue;
       if (STName.startswith(kSPR2TypeName::ImagePrefix)) {
         auto Ty = STName.str();
         auto AccStr = getAccessQualifierFullName(Ty);
-        addAdaptedType(&*Arg, getOrCreateOpaquePtrType(
-                                  M, mapOCLTypeNameToSPIRV(Ty, AccStr)));
+        addAdaptedType(
+            &*Arg,
+            getOrCreateOpaqueStructType(M, mapOCLTypeNameToSPIRV(Ty, AccStr)),
+            SPIRAS_Global);
         Changed = true;
       }
     }
@@ -263,24 +250,22 @@ void OCLTypeToSPIRVBase::adaptArgumentsByMetadata(Function *F) {
   if (!TypeMD)
     return;
   bool Changed = false;
-  auto FT = F->getFunctionType();
-  auto PI = FT->param_begin();
   auto Arg = F->arg_begin();
-  for (unsigned I = 0, E = TypeMD->getNumOperands(); I != E; ++I, ++PI, ++Arg) {
+  for (unsigned I = 0, E = TypeMD->getNumOperands(); I != E; ++I, ++Arg) {
     auto OCLTyStr = getMDOperandAsString(TypeMD, I);
-    auto NewTy = *PI;
-    if (OCLTyStr == OCL_TYPE_NAME_SAMPLER_T && !NewTy->isStructTy()) {
-      addAdaptedType(&(*Arg), getSamplerType(M));
+    if (OCLTyStr == OCL_TYPE_NAME_SAMPLER_T) {
+      addAdaptedType(&(*Arg), getSamplerStructType(M), SPIRAS_Constant);
       Changed = true;
-    } else if (isPointerToOpaqueStructType(NewTy)) {
-      auto STName = NewTy->getPointerElementType()->getStructName();
-      if (STName.startswith(kSPR2TypeName::ImagePrefix)) {
-        auto Ty = STName.str();
+    } else if (OCLTyStr.startswith("image") && OCLTyStr.endswith("_t")) {
+      auto Ty = (Twine("opencl.") + OCLTyStr).str();
+      if (StructType::getTypeByName(F->getContext(), Ty)) {
         auto AccMD = F->getMetadata(SPIR_MD_KERNEL_ARG_ACCESS_QUAL);
         assert(AccMD && "Invalid access qualifier metadata");
         auto AccStr = getMDOperandAsString(AccMD, I);
-        addAdaptedType(&(*Arg), getOrCreateOpaquePtrType(
-                                    M, mapOCLTypeNameToSPIRV(Ty, AccStr)));
+        addAdaptedType(
+            &(*Arg),
+            getOrCreateOpaqueStructType(M, mapOCLTypeNameToSPIRV(Ty, AccStr)),
+            SPIRAS_Global);
         Changed = true;
       }
     }
@@ -317,18 +302,21 @@ void OCLTypeToSPIRVBase::adaptArgumentsByMetadata(Function *F) {
 // opencl data type x and access qualifier y, and use opencl.image_x.y to
 // represent image_x type with access qualifier y.
 //
-Type *OCLTypeToSPIRVBase::getAdaptedType(Value *V) {
-  auto Loc = AdaptedTy.find(V);
-  if (Loc != AdaptedTy.end())
-    return Loc->second;
-
-  if (auto F = dyn_cast<Function>(V))
-    return F->getFunctionType();
-  return V->getType();
+std::pair<Type *, Type *>
+OCLTypeToSPIRVBase::getAdaptedArgumentType(Function *F, unsigned ArgNo) {
+  Value *Arg = F->getArg(ArgNo);
+  auto Loc = AdaptedTy.find(Arg);
+  if (Loc == AdaptedTy.end())
+    return {nullptr, nullptr};
+  Type *PointeeTy = Loc->second.first;
+  Type *PointerTy = PointerType::get(PointeeTy, Loc->second.second);
+  return {PointerTy, PointeeTy};
 }
 
 } // namespace SPIRV
 
+AnalysisKey OCLTypeToSPIRVPass::Key;
+
 INITIALIZE_PASS(OCLTypeToSPIRVLegacy, "cltytospv", "Adapt OCL types for SPIR-V",
                 false, true)
 
