@@ -291,10 +291,12 @@ _SPIRV_OP(ImageSparseTexelsResident, 316)
 _SPIRV_OP(NoLine, 317)
 _SPIRV_OP(AtomicFlagTestAndSet, 318)
 _SPIRV_OP(AtomicFlagClear, 319)
+_SPIRV_OP(SizeOf, 321)
 _SPIRV_OP(TypePipeStorage, 322)
 _SPIRV_OP(ConstantPipeStorage, 323)
 _SPIRV_OP(CreatePipeFromPipeStorage, 324)
 _SPIRV_OP(ModuleProcessed, 330)
+_SPIRV_OP(ExecutionModeId, 331)
 _SPIRV_OP(DecorateId, 332)
 _SPIRV_OP(GroupNonUniformElect, 333)
 _SPIRV_OP(GroupNonUniformAll, 334)
@@ -328,12 +330,23 @@ _SPIRV_OP(GroupNonUniformBitwiseXor, 361)
 _SPIRV_OP(GroupNonUniformLogicalAnd, 362)
 _SPIRV_OP(GroupNonUniformLogicalOr, 363)
 _SPIRV_OP(GroupNonUniformLogicalXor, 364)
+_SPIRV_OP(CopyLogical, 400)
+_SPIRV_OP(PtrEqual, 401)
+_SPIRV_OP(PtrNotEqual, 402)
+_SPIRV_OP(PtrDiff, 403)
+_SPIRV_OP(GroupNonUniformRotateKHR, 4431)
 _SPIRV_OP(SDotKHR, 4450)
 _SPIRV_OP(UDotKHR, 4451)
 _SPIRV_OP(SUDotKHR, 4452)
 _SPIRV_OP(SDotAccSatKHR, 4453)
 _SPIRV_OP(UDotAccSatKHR, 4454)
 _SPIRV_OP(SUDotAccSatKHR, 4455)
+_SPIRV_OP(TypeCooperativeMatrixKHR, 4456)
+_SPIRV_OP(CooperativeMatrixLoadKHR, 4457)
+_SPIRV_OP(CooperativeMatrixStoreKHR, 4458)
+_SPIRV_OP(CooperativeMatrixMulAddKHR, 4459)
+_SPIRV_OP(CooperativeMatrixLengthKHR, 4460)
+_SPIRV_OP(ReadClockKHR, 5056)
 _SPIRV_OP(SubgroupShuffleINTEL, 5571)
 _SPIRV_OP(SubgroupShuffleDownINTEL, 5572)
 _SPIRV_OP(SubgroupShuffleUpINTEL, 5573)
@@ -353,6 +366,8 @@ _SPIRV_OP(AtomicFMinEXT, 5614)
 _SPIRV_OP(AtomicFMaxEXT, 5615)
 _SPIRV_OP(AssumeTrueKHR, 5630)
 _SPIRV_OP(ExpectKHR, 5631)
+_SPIRV_OP(DecorateString, 5632)
+_SPIRV_OP(MemberDecorateString, 5633)
 _SPIRV_OP(VmeImageINTEL, 5699)
 _SPIRV_OP(TypeVmeImageINTEL, 5700)
 _SPIRV_OP(TypeAvcImePayloadINTEL, 5701)
@@ -522,6 +537,9 @@ _SPIRV_OP(ArbitraryFloatPowINTEL, 5880)
 _SPIRV_OP(ArbitraryFloatPowRINTEL, 5881)
 _SPIRV_OP(ArbitraryFloatPowNINTEL, 5882)
 _SPIRV_OP(LoopControlINTEL, 5887)
+_SPIRV_OP(AliasDomainDeclINTEL, 5911)
+_SPIRV_OP(AliasScopeDeclINTEL, 5912)
+_SPIRV_OP(AliasScopeListDeclINTEL, 5913)
 _SPIRV_OP(FixedSqrtINTEL, 5923)
 _SPIRV_OP(FixedRecipINTEL, 5924)
 _SPIRV_OP(FixedRsqrtINTEL, 5925)
@@ -543,3 +561,19 @@ _SPIRV_OP(TypeBufferSurfaceINTEL, 6086)
 _SPIRV_OP(TypeStructContinuedINTEL, 6090)
 _SPIRV_OP(ConstantCompositeContinuedINTEL, 6091)
 _SPIRV_OP(SpecConstantCompositeContinuedINTEL, 6092)
+_SPIRV_OP(ControlBarrierArriveINTEL, 6142)
+_SPIRV_OP(ControlBarrierWaitINTEL, 6143)
+_SPIRV_OP(Subgroup2DBlockLoadINTEL, 6231)
+_SPIRV_OP(Subgroup2DBlockLoadTransformINTEL, 6232)
+_SPIRV_OP(Subgroup2DBlockLoadTransposeINTEL, 6233)
+_SPIRV_OP(Subgroup2DBlockPrefetchINTEL, 6234)
+_SPIRV_OP(Subgroup2DBlockStoreINTEL, 6235)
+_SPIRV_OP(SubgroupMatrixMultiplyAccumulateINTEL, 6237)
+_SPIRV_OP(GroupIMulKHR, 6401)
+_SPIRV_OP(GroupFMulKHR, 6402)
+_SPIRV_OP(GroupBitwiseAndKHR, 6403)
+_SPIRV_OP(GroupBitwiseOrKHR, 6404)
+_SPIRV_OP(GroupBitwiseXorKHR, 6405)
+_SPIRV_OP(GroupLogicalAndKHR, 6406)
+_SPIRV_OP(GroupLogicalOrKHR, 6407)
+_SPIRV_OP(GroupLogicalXorKHR, 6408)
